import rb
import gobject
import dbus
import dbus.glib
import gconf
import gtk

ui_xml = """
<ui>
  <menubar name="MenuBar">
    <menu name="ControlMenu" action="Control">
      <menuitem name="PauseWhenAwayMenu" action="PauseWhenAway"/>
    </menu>
  </menubar>
</ui>
"""

class SessionPresenceManagerPlugin(rb.Plugin):

    def __init__(self):

        rb.Plugin.__init__(self)

    def activate(self, shell):

        self.shell = shell
        self.gconf_client = gconf.client_get_default()
        self.paused_by_us = False
        
        session_bus = dbus.SessionBus()
        session_bus.add_signal_receiver(self.on_status_changed_cb,
                                        signal_name="StatusChanged",
                                        dbus_interface="com.weej.SessionPresenceManager",
                                        named_service=None,
                                        path="/com/weej/SessionPresenceManager")
                                        
        pause_when_away_action = gtk.ToggleAction("PauseWhenAway",
                                                  "Pause When Away",
                                                  "Automatically pauses Rhythmbox when you are away",
                                                  None)
        pause_when_away_action.set_active(self.get_pause_when_away())
        pause_when_away_action.connect("toggled", self.on_pause_when_away_action_toggled_cb)
        
        self.action_group = gtk.ActionGroup("SessionPresenceManagerPluginActions")
        self.action_group.add_action(pause_when_away_action)

        ui_manager = shell.get_player().get_property('ui-manager')
        ui_manager.insert_action_group(self.action_group, 0)

        self.ui_merge_id = ui_manager.add_ui_from_string(ui_xml)
        ui_manager.ensure_update()

    def deactivate(self, shell):
    
        ui_manager = shell.get_player().get_property('ui-manager')
        ui_manager.remove_ui(self.ui_merge_id)
        ui_manager.remove_action_group(self.action_group)
        ui_manager.ensure_update()

        del self.ui_merge_id
        del self.action_group
    
        del self.paused_by_us
        del self.gconf_client
        del self.shell
        
        # FIXME: remove the signal receiver, somehow

    def get_pause_when_away(self):
    
        return self.gconf_client.get_bool("/apps/rhythmbox/plugins/spm/pause_when_away")
        
    def set_pause_when_away(self, value):
    
        self.gconf_client.set_bool("/apps/rhythmbox/plugins/spm/pause_when_away", value)

    def on_status_changed_cb(self, status):

        player = self.shell.get_player()

        # If we are absent, the player is playing, and this thing is enabled, then pause
        if (status[0][0:1] == ["absent",] and player.get_playing() and self.get_pause_when_away()):
            self.shell.get_player().pause()
            self.paused_by_us = True

        # Else, if we are present, and we paused earlier, then unpause
        elif (status[0][0:1] == ["present",] and self.paused_by_us):
            player.play()
            self.paused_by_us = False

           
    def on_pause_when_away_action_toggled_cb(self, action):
    
        self.set_pause_when_away(action.get_active())

