import gobject
import gtk
import dbus
import dbus.glib


def debug(message):

	return
	print(message)


class Client:

    _spm_iface = None
    status_changed_cb = None

    def __init__(self):

        self._session_bus = dbus.SessionBus()
                                             
    def activate(self):
    
        self._session_bus.add_signal_receiver(self._on_name_owner_changed_cb,
                                              signal_name="NameOwnerChanged",
                                              dbus_interface="org.freedesktop.DBus",
                                              named_service="org.freedesktop.DBus",
                                              path="/org/freedesktop/DBus",
                                              arg0="com.weej.SessionPresenceManager")
                                             
        self._try_connect_to_service()

    def get_status(self):

        return self._get_spm_iface().GetStatus()        
            
    def set_status(self, status):

        return self._get_spm_iface().SetStatus(status)
                                             
    def _try_connect_to_service(self):

        try:
        
            spm = self._session_bus.get_object("com.weej.SessionPresenceManager", "/com/weej/SessionPresenceManager")
            debug("Service is available!")
            self._spm_iface = dbus.Interface(spm, "com.weej.SessionPresenceManager")
            self._spm_iface.connect_to_signal("StatusChanged", self._on_status_changed_cb)
            self._on_status_changed_cb(self.get_status())
            
        except (dbus.DBusException):
        
            # Couldn't connect for some reason.
            debug("Service currently unavailable.")
            self._spm_iface = None

    def _get_spm_iface(self):

        if (self._spm_iface):
            return self._spm_iface
        else:
            raise Exception()
            
    def _on_name_owner_changed_cb(self, bus_name, old_owner, new_owner):

        self._try_connect_to_service()
            
    def _on_status_changed_cb(self, status):

        if (self.status_changed_cb):
            self.status_changed_cb(status)


class App:

    def __init__(self):
    
        self.spm_client = Client()
        self.spm_client.status_changed_cb = self._on_status_changed_cb

        self.ui = UI(self)
        self.icon = StatusIcon(self)

        self.spm_client.activate()

        try:
            self.ui.update_status(self.spm_client.get_status())
        except (Exception):
            pass
        
    def _on_status_changed_cb(self, status):
    
        self.ui.update_status(status)


class UI:

    def __init__(self, app):
    
        self.app = app
        
        window = gtk.Window()
        window.connect("delete-event", window.hide_on_delete)
        window.set_title("Session Presence")
        window.set_icon_name("stock_person")
        window.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
        window.set_resizable(False)

        layout_box = gtk.VBox()
        button_box = gtk.HButtonBox()
        status_label = gtk.Label()
        
        here_status = (("present",), "Right here! (SPM)")
        busy_status = (("present", "engaged"), "Busy (SPM)")
        away_status = (("absent",), "Gone... (SPM)")
        
        here_button = gtk.Button("Here")
        here_button.set_image(gtk.image_new_from_icon_name("stock_person", gtk.ICON_SIZE_BUTTON))
        here_button.connect("clicked", self.on_button_clicked_cb, here_status)
        
        busy_button = gtk.Button("Busy")
        busy_button.set_image(gtk.image_new_from_icon_name("stock_person", gtk.ICON_SIZE_BUTTON))
        busy_button.connect("clicked", self.on_button_clicked_cb, busy_status)

        away_button = gtk.Button("Away")
        away_button.set_image(gtk.image_new_from_icon_name("stock_person", gtk.ICON_SIZE_BUTTON))
        away_button.connect("clicked", self.on_button_clicked_cb, away_status)
        
        button_box.add(here_button)
        button_box.add(busy_button)
        button_box.add(away_button)

        layout_box.add(status_label)
        layout_box.add(button_box)

        window.add(layout_box)
        
        layout_box.show_all()
        
        self.window = window
        self.here_button = here_button
        self.busy_button = busy_button
        self.away_button = away_button
        self.status_label = status_label        
        
    def show(self):
    
        self.window.present()        
        
    def hide(self):
    
        self.window.hide()
        
    def update_status(self, status):
    
        self.status_label.set_text("%s - \"%s\"" % ("/".join(status[0]), status[1]))
        
    def on_button_clicked_cb(self, button, status):
    
        self.app.spm_client.set_status(status)


class StatusIcon(gtk.StatusIcon):
    
    def __init__(self, app):
    
        self.app = app
        gtk.StatusIcon.__init__(self)
        self.set_from_icon_name("stock_person")
        
        self.connect("activate", self.on_activate_cb)
        
        self.set_visible(True)

    def on_activate_cb(self, icon):
    
        self.app.ui.show()


app = App()

mainloop = gobject.MainLoop()
mainloop.run()
