import gobject
import dbus
import dbus.glib

class App:

    def __init__(self):

        session_bus = dbus.SessionBus()

        gossip = session_bus.get_object("org.gnome.Gossip", "/org/gnome/Gossip")
        self.gossip_iface = dbus.Interface(gossip, "org.gnome.Gossip")
        spm = session_bus.get_object("com.weej.SessionPresenceManager", "/com/weej/SessionPresenceManager")
        self.spm_iface = dbus.Interface(spm, "com.weej.SessionPresenceManager")

        # Connect the StatusChanged signal and immediately trigger an update
        self.spm_iface.connect_to_signal("StatusChanged", self.on_status_changed_cb)
        self.update_status(self.spm_iface.GetStatus())

    def update_status(self, status):

        # Status mappings:
        #
        # SPM status tuple          Gossip status ID
        # -------------------------------------------------
        # ("present", "engaged")    "busy"
        # ("present")               "available"
        # ("absent")                "away"

        if (status[0][0:1] == ["present",]):
            if (status[0][1:2] == ["engaged",]):
                gossip_status = "busy"
            else:
                gossip_status = "available"
        elif (status[0][0:1] == ["absent",]):
            gossip_status = "away"
        else:
            # Just in case we didn't map a status
            gossip_status = "available"
            
        self.gossip_iface.SetPresence(gossip_status, status[1])

    def on_status_changed_cb(self, status):

        self.update_status(status)


app = App()

mainloop = gobject.MainLoop()
mainloop.run()
